@@ -64,6 +64,39 @@ equivalent to \f(CWtext\fP or ".CW text"
 Remark: The CW face is not available for all groff devices. It is not
 recommended to use it. It is provided for your convenience.
 
+=head2 Automatic characters transliteration
+
+Po4a automatically transliterate some characters to ease the translation
+or the review of the translation.
+Here is the list of the transliterations:
+
+=over
+
+=item hyphens
+
+Hyphens (-) and minus signs (\-) in man pages are all transliterated
+as simple dashes (-) in the PO file.  Then all dash are transliterated into
+roff minus signs (\-) when the translation is inserted into the output
+document.
+
+Translators can force an hyphen by using the roff glyph '\[hy]' in their
+translations.
+
+=item non-breaking spaces
+
+Translators can use non-breaking spaces in their translations.  These
+non-breaking spaces (0xA0) will be transliterated into a roff
+non-breaking space ('\ ').
+
+=item quotes transliterations
+
+`` and '' are respectively tranliterated into \*(lq and \*(rq.
+
+To avoid these transliterations, translators can insert a zero width roff
+character (i.e., using `\&` or '\&' respectively).
+
+=back
+
 =head2 Putting 'E<lt>' and 'E<gt>' in translations
 
 Since these chars are used to delimit parts under font modification, you
@@ -81,26 +114,6 @@ These are this module's particular options:
 Activate debugging for some internal mechanisms of this module.
 Use the source to see which parts can be debugged.
 
-=item B<with-non_breaking_spaces> (boolean)
-
-Allow po4a to generate non breaking spaces in the PO file (when needed).
-
-Anyway, users can use non breaking spaces in the PO and are always converted to
-the roff non breaking space ('\ ').
-
-This option is disabled by default because non breaking spaces are not present
-in all character set.
-
-=item B<with-hyphen_to_minus> (boolean)
-
-Allow the automatic convertion of hyphens (in roff: '-') to minus signs ('\-').
-When enable, translators will only see/use '-' instead of '\\-' in their PO
-files.
-
-This option is enabled by default, because man pages autors and translators
-always forget to use '\-' instead of '-' (and it doesn't harm to have a minus
-sign instead of an hyphen).
-
 =back
 
 =head1 AUTHORING MAN PAGES COMPLIANT WITH PO4A::MAN
@@ -280,6 +293,12 @@ my %macro; # hash of known macro, with parsing sub. See end of this file
 # .    - a single char (e.g. B, I, R, P, 1, 2, 3, 4, etc.)
 my $FONT_RE = "\\\\f(?:\\[[^\\]]*\\]|\\(..|[^\\(\\[])";
 
+# Variable used to identify non breaking spaces.
+# These non breaking spaces are used to ease the parsing, and a
+# translator can use them in her translation (and they will be translated
+# into the groff non-breaking space.
+my $nbs;
+
 #########################
 #### DEBUGGING STUFF ####
 #########################
@@ -291,21 +310,19 @@ my %debug=('splitargs' => 0, # see how macro args are separated
 
 
 ######## CONFIG #########
-my %transliterations = (
-    'nbs'   => 0,     # allow generating non breaking spaces?
-    'minus' => 1      # force using minus signs instead of hyphens?
-    );
 sub initialize {
     my $self = shift;
     my %options = @_;
 
-    $self->{options}{'with-non_breaking_spaces'}='';
-    $self->{options}{'with-hyphen_to_minus'}='';
     $self->{options}{'debug'}='';
+    $self->{options}{'verbose'}='';
+
+    # reset the debug options
+    %debug = ();
 
     foreach my $opt (keys %options) {
-        if ($options{$opt}) {
-            die wrap_mod("po4a::tex",
+        if (defined $options{$opt}) {
+            die wrap_mod("po4a::man",
                          dgettext("po4a", "Unknown option: %s"), $opt)
                 unless exists $self->{options}{$opt};
             $self->{options}{$opt} = $options{$opt};
@@ -317,16 +334,10 @@ sub initialize {
             $debug{$_} = 1;
         }
     }
-
-    if (defined $options{'with-non_breaking_spaces'}) {
-        $transliterations{'nbs'} = $options{'with-non_breaking_spaces'};
-    }
-
-    if (defined $options{'with-hyphen_to_minus'}) {
-        $transliterations{'minus'} = $options{'with-hyphen_to_minus'};
-    }
 }
 
+my @comments = ();
+my @next_comments = ();
 # This function returns the next line of the document being parsed
 # (and its reference).
 # It overload the Transtractor shiftline to handle:
@@ -339,6 +350,7 @@ sub initialize {
 sub shiftline {
     my $self = shift;
     # call Transtractor's shiftline
+NEW_LINE:
     my ($line,$ref) = $self->SUPER::shiftline();
 
     if (!defined $line) {
@@ -360,6 +372,61 @@ sub shiftline {
     $line =~ s/\\\././g;
 
     chomp $line;
+    if ($line =~ m/^(.*?)(?:(?<!\\)\\"(.*))$/) {
+        my ($l, $c) = ($1, $2);
+        $line = $l;
+        # Check for comments indicating that the file was generated.
+        if ($c =~ /Pod::Man/) {
+            warn wrap_mod("po4a::man", dgettext("po4a", "This file was generated with Pod::Man. Translate the pod file with the pod module of po4a."));
+            exit 254;
+        } elsif ($c =~ /generated by help2man/)    {
+            warn wrap_mod("po4a::man", dgettext("po4a", "This file was generated with help2man. Translate the source file with the regular gettext."));
+        } elsif ($c =~ /with docbook-to-man/)      {
+            warn wrap_mod("po4a::man", dgettext("po4a", "This file was generated with docbook-to-man. Translate the source file with the sgml module of po4a."));
+            exit 254;
+        } elsif ($c =~ /generated by docbook2man/) {
+            warn wrap_mod("po4a::man", dgettext("po4a", "This file was generated with docbook2man. Translate the source file with the sgml module of po4a."));
+            exit 254;
+        } elsif ($c =~ /created with latex2man/)   {
+            warn wrap_mod("po4a::man", dgettext("po4a",
+                "This file was generated with %s. ".
+                "You should translate the source file, but continuing anyway."
+                ),"latex2man");
+        } elsif ($c =~ /Generated by db2man.xsl/)  { 
+            warn wrap_mod("po4a::man", dgettext("po4a","This file was generated with db2man.xsl. Translate the source file with the xml module of po4a."));
+            exit 254;
+        } elsif ($c =~ /generated automatically by mtex2man/)  {
+            warn wrap_mod("po4a::man", dgettext("po4a",
+                "This file was generated with %s. ".
+                "You should translate the source file, but continuing anyway."
+                ),"mtex2man");
+        } elsif ($c =~ /THIS FILE HAS BEEN AUTOMATICALLY GENERATED.  DO NOT EDIT./ ||
+                 $c =~ /DO NOT EDIT/i || $c =~ /generated/i) {
+            warn wrap_mod("po4a::man", dgettext("po4a",
+                "This file contains the line '%s'. ".
+                "You should translate the source file, but continuing anyway."
+                ),$l."\\\"".$c);
+        }
+
+        if ($line =~ m/^[.']*$/) {
+            if ($c !~ m/^\s*$/) {
+                # This commentted line may be comment for the next paragraph
+                push @next_comments, $c;
+            }
+            if ($line =~ m/^[.']+$/) {
+                # those lines are ignored
+                # (empty lines are a little bit different)
+                goto NEW_LINE;
+            }
+        } else {
+            push @comments, $c;
+        }
+    } else {
+        # finally, we did not reach the end of the paragraph.  The comments
+        # belong to the current paragraph.
+        push @comments, @next_comments;
+        @next_comments = ();
+    }
     while ($line =~ /\\$/ || $line =~ /^(\.[BI])\s*$/) {
         my ($l2,$r2)=$self->SUPER::shiftline();
         chomp($l2);
@@ -410,6 +477,23 @@ sub shiftline {
     return ($line,$ref);
 }
 
+# Overload Transtractor's pushline.
+# This pushline first push comments (if there are comments for the
+# current line, and the line is not empty), and then push the line.
+sub pushline {
+    my ($self, $line) = (shift, shift);
+    if ($line !~ m/^\s*$/) {
+        # add comments
+        foreach my $c (@comments) {
+            # comments are pushed (maybe at the wrong place).
+            $self->SUPER::pushline(".\\\"$c\n");
+        }
+        @comments = ();
+    }
+
+    $self->SUPER::pushline($line);
+}
+
 # The default unshiftline from Transtractor may fail because shiftline
 # is overloaded
 sub unshiftline {
@@ -505,20 +589,16 @@ sub pre_trans {
     #  * they break when using utf8 (for obscure reasons)
     #  * they forbid the searches, since keybords don't have hyphen key
     #  * they forbid copy/paste, since options need minus sign, not hyphen
-    if ($transliterations{'minus'}) {
-        $str =~ s|\\-|-|sg;
-    }
+    $str =~ s|\\-|-|sg;
+
     # Groff bestiary
     $str =~ s/\\\*\(lq/``/sg;
     $str =~ s/\\\*\(rq/''/sg;
     $str =~ s/\\\(dq/"/sg;
-    # Change groff non-breaking space to ascii one
-    if ($transliterations{'nbs'}) {
-        $str =~ s|\\ |\xA0|sg;
-    } else {
-        # some \xA0 may have been added during the parsing
-        $str =~ s|\xA0|\\ |sg;
-    }
+    
+    # non-breaking spaces
+    # some non-breaking spaces may have been added during the parsing
+    $str =~ s/\Q$nbs/\\ /sg;
 
     print STDERR "$str\n" if ($debug{'pretrans'});
     return $str;
@@ -532,14 +612,12 @@ sub post_trans {
 	if ($debug{'postrans'});
 
     # Post formatting, so that groff see the strange chars
-    if ($transliterations{'minus'}) {
-        $str =~ s|\\-|-|sg; # in case the translator added some of them manually
-        # change hyphens to minus signs
-        # (this shouldn't be done for \s-<number> font size modifiers)
-        # nor on .so/.mso args
-        unless (defined $self->{type} && $self->{type} =~ m/^m?so$/) {
-            $str =~ s/(?<!\\s)-/\\-/sg; # (?<!pattern) means "not preceded by pattern"
-        }
+    $str =~ s|\\-|-|sg; # in case the translator added some of them manually
+    # change hyphens to minus signs
+    # (this shouldn't be done for \s-<number> font size modifiers)
+    # nor on .so/.mso args
+    unless (defined $self->{type} && $self->{type} =~ m/^m?so$/) {
+        $str =~ s/(?<!\\s)-/\\-/sg; # (?<!pattern) means "not preceded by pattern"
     }
 
     # No . or ' on first char, or nroff will think it's a macro
@@ -559,11 +637,14 @@ sub post_trans {
     $str =~ s/\n([.'])/ $1/mg;
 
     # Change ascii non-breaking space to groff one
-    $str =~ s|\xA0|\\ |sg;
+    my $nbs_out = "\xA0";
+    my $enc_length = Encode::from_to($nbs_out, "latin1",
+                                               $self->get_out_charset);
+    $str =~ s/\Q$nbs_out/\\ /sg if defined $enc_length;
     # No nbsp (said "\ " in groff on the last pos of the line, or groff adds
     # an extra space
     $str =~ s/\\ \n/\\ /sg;
-	
+
     # Make sure we compute internal sequences right.
     # think about: B<AZE E<lt> EZA E<gt>>
     while ($str =~ m/^(.*)(CW|[RBI])<(.*)$/s) {
@@ -607,6 +688,7 @@ sub translate {
     return $str if ($str eq "\n");
 
     $str=pre_trans($self,$str,$ref||$self->{ref},$type);
+    $options{'comment'} .= join('\n', @comments);
     # Translate this
     $str = $self->SUPER::translate($str,
 				   $ref||$self->{ref},
@@ -659,6 +741,20 @@ sub parse{
                             # MACRONO: don't wrap because we saw the nf macro. It stays so 
                             #          until the next fi macro.
 
+
+    # change the non-breaking space according to the input document charset
+    $nbs = "\xA0";
+    my $enc_length;
+    if (defined $self->{TT}{'file_in_charset'} and
+        length $self->{TT}{'file_in_charset'})
+    {
+        $enc_length = Encode::from_to($nbs, "latin-1",
+                                           $self->{TT}{'file_in_charset'});
+    }
+    # fall back solution
+    $nbs = "PO4A:VERY_IMPROBABLE_STRING_USEDFOR_NON-BREAKING-SPACES"
+        unless defined $enc_length;
+
   LINE:
     undef $self->{type};
     ($line,$ref)=$self->shiftline();
@@ -681,7 +777,7 @@ sub parse{
 
 	    # Split on spaces for arguments, but not spaces within double quotes
 	    my @args=();
-	    $line =~ s/\\ /\xA0/g; # This is probably not needed
+	    $line =~ s/\\ /$nbs/g; # This is probably not needed
 	    push @args,$arg1;
 	    push @args, splitargs($ref,$arguments);
 
@@ -692,43 +788,6 @@ sub parse{
 		$wrapped_mode = $wrapped_mode eq 'NO' ? 'YES' : $wrapped_mode;
 	    }
 
-	    # Special case: Don't change these lines
-	    # Check for comments indicating that the file was generated.
-	    if ($macro eq '\"' ||
-		$macro eq '"') {
-		if ($line =~ /Pod::Man/) {
-		    warn wrap_mod("po4a::man", dgettext("po4a", "This file was generated with Pod::Man. Translate the pod file with the pod module of po4a."));
-		    exit 254;
-		} elsif ($line =~ /generated by help2man/)    {
-		    warn wrap_mod("po4a::man", dgettext("po4a", "This file was generated with help2man. Translate the source file with the regular gettext."));
-		} elsif ($line =~ /with docbook-to-man/)      {
-		    warn wrap_mod("po4a::man", dgettext("po4a", "This file was generated with docbook-to-man. Translate the source file with the sgml module of po4a."));
-		    exit 254;
-		} elsif ($line =~ /generated by docbook2man/) {
-		    warn wrap_mod("po4a::man", dgettext("po4a", "This file was generated with docbook2man. Translate the source file with the sgml module of po4a."));
-		    exit 254;
-		} elsif ($line =~ /created with latex2man/)   {
-		    warn wrap_mod("po4a::man", dgettext("po4a",
-			"This file was generated with %s. ".
-			"You should translate the source file, but continuing anyway."
-			),"latex2man");
-		} elsif ($line =~ /Generated by db2man.xsl/)  { 
-		    warn wrap_mod("po4a::man", dgettext("po4a","This file was generated with db2man.xsl. Translate the source file with the xml module of po4a."));
-		    exit 254;
-		} elsif ($line =~ /generated automatically by mtex2man/)  {
-		    warn wrap_mod("po4a::man", dgettext("po4a",
-			"This file was generated with %s. ".
-			"You should translate the source file, but continuing anyway."
-			),"mtex2man");
-		} elsif ($line =~ /THIS FILE HAS BEEN AUTOMATICALLY GENERATED.  DO NOT EDIT./ ||
-		         $line =~ /DO NOT EDIT/i || $line =~ /generated/i) {
-		    warn wrap_mod("po4a::man", dgettext("po4a",
-			"This file contains the line '%s'. ".
-			"You should translate the source file, but continuing anyway."
-			),$line);
-		}
-	    }
-
 	    # Special case: Don't change these lines
 	    #  .\"  => comments
 	    #  ."   => comments
@@ -810,6 +869,11 @@ sub parse{
 	    $self->pushline($line."\n");
 	}
 
+	# finally, we did not reach the end of the paragraph.  The comments
+	# belong to the current paragraph.
+	push @comments, @next_comments;
+	@next_comments = ();
+
 	# Reinit the loop
 	($line,$ref)=$self->shiftline();
 	undef $self->{type};
@@ -820,6 +884,14 @@ sub parse{
 	$wrapped_mode = $wrapped_mode eq 'NO' ? 'YES' : $wrapped_mode;
 	$paragraph="";
     }
+
+    # flush the last comments
+    push @comments, @next_comments;
+    @next_comments = @comments;
+    @comments = ();
+    for my $c (@next_comments) {
+	$self->pushline(".\\\"$c\n");
+    }
 } # end of main
 
 
@@ -836,12 +908,15 @@ sub splitargs {
     my @args=();
     my $buffer="";
     my $escaped=0;
+    if (! defined $arguments) {
+        return @args;
+    }
     # change non-breaking space before to ensure that split does what we want
     # We change them back before pushing into the arguments. The one which
     # will be translated will have the same change again (in pre_trans and
     # post_trans), but the ones which won't get translated are not changed
     # anymore. Let's play safe.
-    $arguments =~ s/\\ /\xA0/g;
+    $arguments =~ s/\\ /$nbs/g;
     $arguments =~ s/^ +//;
     foreach my $elem (split (/ +/,$arguments)) {
         print STDERR ">>Seen $elem(buffer=$buffer;esc=$escaped)\n"
@@ -858,8 +933,8 @@ sub splitargs {
                 print STDERR "End of quote, with stuff after it\n"
                     if ($debug{'splitargs'});
                 my ($a,$b)=($1,$2);
-                $a =~ s/\xA0/\\ /g;
-                $b =~ s/\xA0/\\ /g;
+                $a =~ s/\Q$nbs/\\ /g;
+                $b =~ s/\Q$nbs/\\ /g;
                 push @args,$a;
                 push @args,$b;
                 $buffer = "";
@@ -867,7 +942,7 @@ sub splitargs {
                 print STDERR "End of a quote\n"
                     if ($debug{'splitargs'});
                 my $a = $1;
-                $a =~ s/\xA0/\\ /g;
+                $a =~ s/\Q$nbs/\\ /g;
                 push @args,$a;
                 $buffer = "";
             } elsif ($escaped) {
@@ -875,9 +950,9 @@ sub splitargs {
                     if ($debug{'splitargs'});
                 unless(length($elem)){
                     die wrap_ref_mod($ref, "po4a::man", dgettext("po4a",
-			"Escaped space at the end of macro arg. With high probability, it won't do the trick with po4a (because of wrapping). You may want to remove it and use the .nf/.fi groff macro to control the wrapping."));
+                        "Escaped space at the end of macro arg. With high probability, it won't do the trick with po4a (because of wrapping). You may want to remove it and use the .nf/.fi groff macro to control the wrapping."));
                 }
-                $buffer =~ s/\xA0/\\ /g;
+                $buffer =~ s/\Q$nbs/\\ /g;
                 push @args,$buffer;
                 $buffer = "";
                 $escaped = 0;
@@ -888,7 +963,7 @@ sub splitargs {
             my $a = $1;
             # Inside a quoted argument, "" indicates a double quote
             $a =~ s/""/\\(dq/g;
-            $a =~ s/\xA0/\\ /g;
+            $a =~ s/\Q$nbs/\\ /g;
             push @args,$a;
         } elsif ($elem =~ m/^"/) { #") {
             print STDERR "Begin of a quoting arg\n"
@@ -911,7 +986,7 @@ sub splitargs {
         # Inside a quoted argument, "" indicates a double quote
         while ($buffer =~ s/^"([^"]*?)""/"$1\\(dq/) {}
         $buffer=~ s/"//g;
-        $buffer =~ s/\xA0/\\ /g;
+        $buffer =~ s/\Q$nbs/\\ /g;
         push @args,$buffer;
     }
     if ($debug{'splitargs'}) {
