@@ -10,7 +10,7 @@
 # Copyright (c) 1995 by David Megginson <dmeggins@aix1.uottawa.ca>
 # 
 # The adaptation for po4a was done by Denis Barbier <barbier@linuxfr.org>,
-# Martin Quinson <martin.quinson@tuxfamily.org> and others.
+# Martin Quinson (mquinson#debian.org) and others.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -91,7 +91,7 @@ Proceed even if the DTD is unknown.
 By default, msgids containing only one entity (like '&version;') are skipped
 for the translator comfort. Activating this option prevents this
 optimisation. It can be useful if the document contains a construction like
-"<title>&Acute;</title>", even if I doubt such things to ever happen...
+"<title>&Aacute;</title>", even if I doubt such things to ever happen...
 
 =back
 
@@ -105,24 +105,24 @@ same. But there is still some problems:
 =item * 
 
 the error output of nsgmls is redirected to /dev/null, which is clearly
-bad. I dunno how to avoid that.
+bad. I don't know how to avoid that.
 
-The problem is that I have to "protect" the conditionnal inclusion (ie, the
-C<E<lt>! [ %blah [> and C<]]E<gt>> stuff) from nsgml, because in the other
-case, nsgmls eat them, and I dunno how to restore them in the final
-document. To prevent that, I rewrite them to C<{PO4A-beg-blah}> and
+The problem is that I have to "protect" the conditional inclusions (ie, the
+C<E<lt>! [ %foo [> and C<]]E<gt>> stuff) from nsgmls. Otherwise
+nsgmls eats them, and I don't know how to restore them in the final
+document. To prevent that, I rewrite them to C<{PO4A-beg-foo}> and
 C<{PO4A-end}>. 
 
 The problem with this is that the C<{PO4A-end}> and such I add are valid in
 the document (not in a E<lt>pE<gt> tag or so).
 
 Everything works well with nsgmls's output redirected that way, but it will
-prevent us to detect that the document is badly formated.
+prevent us to detect that the document is badly formatted.
 
 =item *
 
 It does work only with the debiandoc and docbook dtd. Adding support for a
-new dtd should be very easy. The mecanism is the same for all dtd, you just
+new dtd should be very easy. The mechanism is the same for every dtd, you just
 have to give a list of the existing tags and some of their characteristics.
 
 I agree, this needs some more documentation, but it is still considered as
@@ -131,7 +131,7 @@ beta, and I hate to document stuff which may/will change.
 =item *
 
 Warning, support for dtds is quite experimental. I did not read any
-reference manual to find the definition of all tags. I did add tag
+reference manual to find the definition of every tag. I did add tag
 definition to the module 'till it works for some documents I found on the
 net. If your document use more tags than mine, it won't work. But as I said
 above, fixing that should be quite easy.
@@ -140,7 +140,7 @@ I did test docbook against the SAG (System Administrator Guide) only, but
 this document is quite big, and should use most of the docbook
 specificities. 
 
-For debiandoc, I tested some of the manual of the DDP, but not all yet.
+For debiandoc, I tested some of the manuals from the DDP, but not all yet.
 
 =item * 
 
@@ -148,7 +148,7 @@ In case of file inclusion, string reference of messages in po files (ie,
 lines like C<#: en/titletoc.sgml:9460>) will be wrong. 
 
 This is because I preprocess the file to protect the conditional inclusion
-(ie, the C<E<lt>! [ %blah [> and C<]]E<gt>> stuff) and some entities (like
+(ie, the C<E<lt>! [ %foo [> and C<]]E<gt>> stuff) and some entities (like
 &version;) from nsgmls because I want them verbatim to the generated
 document. For that, I make a temp copy of the input file and do all the
 changes I want to this before passing it to nsgmls for parsing.
@@ -175,11 +175,12 @@ use vars qw(@ISA @EXPORT);
 @EXPORT = qw();
 
 use Locale::Po4a::TransTractor;
+use Locale::Po4a::Common;
 use Locale::gettext qw(dgettext);
 
 eval qq{use SGMLS};
 if ($@) {
-  die "po4a::sgml: ".dgettext("po4a","The needed module SGMLS.pm was not found and needs to be installed. It can be found on the CPAN, in package libsgmls-perl on debian, etc.")."\n";
+  die wrap_mod("po4a::sgml", dgettext("po4a","The needed module SGMLS.pm was not found and needs to be installed. It can be found on the CPAN, in package libsgmls-perl on debian, etc."));
 }
 
 use File::Temp;
@@ -211,7 +212,7 @@ sub initialize {
     
     foreach my $opt (keys %options) {
 	if ($options{$opt}) {
-	    die sprintf("po4a::sgml: ".dgettext ("po4a","Unknown option: %s"), $opt)."\n" unless exists $self->{options}{$opt};
+	    die wrap_mod("po4a::sgml", dgettext ("po4a", "Unknown option: %s"), $opt) unless exists $self->{options}{$opt};
 	    $self->{options}{$opt} = $options{$opt};
 	}
     }
@@ -245,14 +246,14 @@ sub translate {
     # don't translate entries composed of one entity
     if ( (($string =~ /^&[^;]*;$/) || ($options{'wrap'} && $string =~ /^\s*&[^;]*;\s*$/))
 	 && !($self->{options}{'include-all'}) ){
-	warn sprintf("po4a::sgml: ".dgettext("po4a","msgid skipped to help translators (contains only an entity)"), $string)."\n"
-	       unless $self->verbose() <= 0;
+	warn wrap_mod("po4a::sgml", dgettext("po4a", "msgid skipped to help translators (contains only an entity)"), $string)
+	    unless $self->verbose() <= 0;
 	return $string;
     }
     # don't translate entries composed of tags only
     if ( $string =~ /^(((<[^>]*>)|\s)*)$/
 	 && !($self->{options}{'include-all'}) ) {
-	warn sprintf("po4a::sgml: ".dgettext("po4a","msgid skipped to help translators (contains only tags)"), $string)."\n"
+	warn wrap_mod("po4a::sgml", dgettext("po4a", "msgid skipped to help translators (contains only tags)"), $string)
 	       unless $self->verbose() <= 0;
 	return $string;
     }
@@ -296,17 +297,17 @@ sub parse_file {
     #   - protect optional inclusion marker (ie, "<![ %str [" and "]]>")
     #   - protect entities from expansion (ie "&release;")
     open (IN,"<$filename") 
-	|| die sprintf(dgettext("po4a","Can't open %s: %s"),$filename,$!)."\n";
+	|| die wrap_mod("po4a::sgml", dgettext("po4a", "Can't open %s: %s"), $filename, $!);
     my $origfile="";
     while (<IN>) {
 	$origfile .= $_;
     }
-    close IN || die sprintf("po4a::sgml: ".dgettext("po4a","can't close %s: %s"),$filename,$!)."\n";
+    close IN || die wrap_mod("po4a::sgml", dgettext("po4a", "Can't close %s: %s"), $filename, $!);
     # Detect the XML pre-prolog
     if ($origfile =~ s/^(\s*<\?xml[^?]*\?>)//) {
-	warn sprintf(dgettext("po4a",
-		"po4a::sgml: Trying to handle a XML document as a SGML one.\n".
-		"po4a::sgml: Feel lucky if it works, help us implementing a proper XML backend if it does not."),$filename)."\n"
+	warn wrap_mod("po4a::sgml", dgettext("po4a",
+		"Trying to handle a XML document as a SGML one. ".
+		"Feel lucky if it works, help us implementing a proper XML backend if it does not."), $filename)
 	  unless $self->verbose() <= 0;
 	$xmlprolog=$1;
     }
@@ -317,20 +318,23 @@ sub parse_file {
 	my $pos = 0;  # where in the document (in chars) while detecting prolog boundaries
 	
 	unless ($prolog =~ s/^(.*<!DOCTYPE).*$/$1/is) {
-	    die sprintf(dgettext("po4a",
-	    	"po4a::sgml: This file is not a master SGML document (no DOCTYPE).\n".
-		"po4a::sgml: It may be a file to be included by another one, in which case\n".
-		"po4a::sgml: it should not be passed to po4a directly. Text from included\n".
-		"po4a::sgml: files is extracted/translated when handling the master file\n".
-		"po4a::sgml: including them."), $filename)."\n";
+	    die wrap_mod("po4a::sgml", dgettext("po4a",
+	    	"This file is not a master SGML document (no DOCTYPE). ".
+		"It may be a file to be included by another one, in which case it should not be passed to po4a directly. Text from included files is extracted/translated when handling the master file including them."));
 	}
 	$pos += length($prolog);
 	$lvl=1;
 	while ($lvl != 0) {
-	    # Eat comments in the prolog, since it may be some '>' or '<' in them
+	    # Eat comments in the prolog, since it may be some '>' or '<' in them.
 	    if ($origfile =~ m/^.{$pos}?(<!--.*?-->)/s) {
 		print "Found a comment in the prolog: $1\n" if ($debug{'generic'});
 		$pos += length($1);
+		# take care of the line numbers
+		my @a = split(/\n/,$1);
+		shift @a; # nb line - 1
+		while (defined(shift @a)) {
+		    $prolog .= "\n";
+		}
 		next;
 	    }
 	    # Search the closing '>'
@@ -397,7 +401,8 @@ sub parse_file {
 	                                    "table tbody textobject tgroup thead tip toc ".
 	                                    "variablelist varlistentry videoobject ".
 	                                    "warning",
-			     "verbatim"  => "address programlisting literallayout refentrytitle screen title",
+			     "verbatim"  => "address holder literallayout option programlisting ".
+	                                    "refentrytitle refname refpurpose screen title",
 			     "ignore"    => "action affiliation anchor application author authorinitials ".
 	                                    "command citation citerefentry citetitle classname co computeroutput constant corpauthor ".
 	                                    "database ".
@@ -424,16 +429,14 @@ sub parse_file {
 
     } else {
 	if ($self->{options}{'force'}) {
-	    warn "po4a::sgml: ".dgettext("po4a","DTD of this file is unknown, but proceeding as requested.")."\n";
+	    warn wrap_mod("po4a::sgml", dgettext("po4a", "DTD of this file is unknown, but proceeding as requested."));
 	    $self->set_tags_kind();
 	} else {
-	    die sprintf("po4a::sgml: ".dgettext("po4a",
-	    	"DTD of this file is unknown. (supported: debiandoc, docbook).\n".
-		"The prolog follows:\n%s"),
-	  	                $filename,$prolog)."\n";
+	    die wrap_mod("po4a::sgml", dgettext("po4a",
+		"DTD of this file is unknown. (supported: debiandoc, docbook). The prolog follows:")."\n$prolog");
 	}
     }
-    
+
     # Prepare the reference indirection stuff
     my @refs;
     my @lines = split(/\n/, $origfile);
@@ -463,8 +466,8 @@ sub parse_file {
 	    my $filename=$3;
 	    $prolog = $1.$4;
 	    (-e $filename && open IN,"<$filename")  ||
-	      die sprintf("po4a::sgml: ".dgettext("po4a","Can't open %s (content of entity %s%s;): %s"),
-		  $filename,'%',$key,$!)."\n";
+	      die wrap_mod("po4a::sgml", dgettext("po4a", "Can't open %s (content of entity %s%s;): %s"),
+		  $filename, '%', $key, $!);
 	    local $/ = undef;
 	    $prologentincl{$key} = <IN>;
 	    close IN;
@@ -499,16 +502,16 @@ sub parse_file {
 	    }
 	}
     }
-    # Unprotect undefined inclusions
+    # Unprotect undefined inclusions, and die of them
     $prolog =~ s/{PO4A-percent}/%/sg;
     if ($prolog =~ /%([^;\s]*);/) {
-       die sprintf("po4a::sgml: ".dgettext("po4a","unrecognized prolog inclusion entity: %%%s;")."\n",$1);
+       die wrap_mod("po4a::sgml", dgettext("po4a","unrecognized prolog inclusion entity: %%%s;"), $1);
     }
-    # Protect &entities; (but the ones asking for a file inclusion)
+    # Protect &entities; (all but the ones asking for a file inclusion)
     #   search the file inclusion entities
     my %entincl;
     my $searchprolog=$prolog;
-    while ($searchprolog =~ /(.*?)<!ENTITY\s(\S*)\s*SYSTEM\s*"([^>"]*)">(.*)$/is) {#})"{
+    while ($searchprolog =~ /(.*?)<!ENTITY\s(\S*)\s*SYSTEM\s*"([^>"]*)">(.*)$/is) {  #})"{
 	print STDERR "Seen the entity of inclusion $2 (=$3)\n"
 	  if ($debug{'entities'});
 	my $key = $2;
@@ -517,8 +520,8 @@ sub parse_file {
 	$entincl{$key}{'filename'}=$filename;
 	# Preload the content of the entity
 	(-e $filename && open IN,"<$filename")  ||
-	  die sprintf("po4a::sgml: ".dgettext("po4a","Can't open %s (content of entity %s%s;): %s"),
-	      $filename,'&',$key,$!)."\n";
+	  die wrap_mod("po4a::sgml", dgettext("po4a", "Can't open %s (content of entity %s%s;): %s"),
+	      $filename, '&', $key, $!);
 	local $/ = undef;
 	$entincl{$key}{'content'} = <IN>;
 	close IN;
@@ -528,7 +531,7 @@ sub parse_file {
 	  if ($debug{'entities'});
     }
 
-    #   Change the entities in the file
+    #   Change the entities including files in the document
     while ($origfile =~ /^(.*?)&([^;\s]*);(.*)$/s) {
 	if (defined $entincl{$2}) {
 	    my ($begin,$key,$end)=($1,$2,$3);
@@ -568,8 +571,7 @@ sub parse_file {
 	    print STDERR "preserve $2\n" if ($debug{'entities'});
 	}
     }
-    #   Reput the entities of inclusion in place
-    $origfile =~ s/{PO4A-keep-amp}/&/g;
+
     if ($debug{'refs'}) {
 	print "XX Resulting shifts\n";
 	for (my $i=0; $i<scalar @refs; $i++) {
@@ -577,16 +579,16 @@ sub parse_file {
 	}
     }
     
-    my ($tmpfh,$tmpfile)=File::Temp->tempfile("po4a-sgml-XXXX",
+    my ($tmpfh,$tmpfile)=File::Temp->tempfile("po4asgml-XXXX",
 					      DIR    => "/tmp",
 					      UNLINK => 0);
     print $tmpfh $origfile;
-    close $tmpfh || die sprintf(dgettext("po4a","Can't close tempfile: %s"),$!)."\n";
+    close $tmpfh || die wrap_mod("po4a::sgml", dgettext("po4a", "Can't close tempfile: %s"), $!);
 
     my $cmd="cat $tmpfile|nsgmls -l -E 0 2>/dev/null|";
     print STDERR "CMD=$cmd\n" if ($debug{'generic'});
 
-    open (IN,$cmd) || die sprintf(dgettext("po4a","Can't run nsgmls: %s"),$!)."\n";
+    open (IN,$cmd) || die wrap_mod("po4a::sgml", dgettext("po4a", "Can't run nsgmls: %s"), $!);
 
     # The kind of tags
     my (%translate,%empty,%verbatim,%indent,%exist);
@@ -630,7 +632,7 @@ sub parse_file {
         while ($prolog =~ m/^(.*?)<!ENTITY\s+(\S*)\s+"([^"]*)">(.*)$/is) { #" ){ 
 	   $self->pushline($1) if length($1);
 	   $self->pushline("<!ENTITY $2 \"".$self->translate($3,"","definition of entity \&$2;")."\">");
-	   warn "Seen text entity $2" if ($debug{'entities'});
+	   warn "Seen text entity $2\n" if ($debug{'entities'});
 	   $prolog = $4;
 	}
         $self->pushline($post."\n") if (length($post));
@@ -660,8 +662,8 @@ sub parse_file {
 	my $type;
 	
 	if ($event->type eq 'start_element') {
-	    die sprintf("po4a::sgml: ".dgettext("po4a","%s: Unknown tag %s"),
-			$refs[$parse->line],$event->data->name)."\n" 
+	    die wrap_ref_mod($ref, "po4a::sgml", dgettext("po4a", "Unknown tag %s"),
+			$event->data->name)
 		unless $exist{$event->data->name};
 	    
 	    $lastchar = ">";
@@ -724,9 +726,8 @@ sub parse_file {
 		$buffer="";
 		push @open,$tag;
 	    } elsif ($indent{$event->data->name()}) {
-		die sprintf(dgettext("po4a",
-		    "Closing tag for a translation container missing before %s, at %s"
-				    ),$tag,$ref)."\n"
+		die wrap_ref_mod($ref, "po4a::sgml", dgettext("po4a",
+		    "Closing tag for a translation container missing before %s"),$tag)
 		    if (scalar @open);
 	    }
 
@@ -773,9 +774,8 @@ sub parse_file {
 		    push @open,$tag;
 		}
 	    } elsif ($indent{$event->data->name()}) {
-		die sprintf(dgettext("po4a",
-           "Closing tag for a translation container missing before %s, at %s"
-				    ),$tag,$ref)."\n"
+		die wrap_ref_mod($ref, "po4a::sgml", dgettext("po4a",
+           "Closing tag for a translation container missing before %s"), $tag)
 		    if (scalar @open);
 	    }
 
@@ -830,9 +830,8 @@ sub parse_file {
 	}
 
 	else {
-	    die sprintf("po4a::sgml: ".dgettext("po4a","%s: Unknown SGML event type: %s"),
-			$refs[$parse->line],$event->type)."\n";
-	    
+	    die wrap_ref_mod($refs[$parse->line], "po4a::sgml", dgettext("po4a","Unknown SGML event type: %s"), $event->type);
+
 	}
     }
 				
@@ -881,12 +880,12 @@ SGMLS and NSGMLS parsers) which was:
 The adaptation for po4a was done by:
 
  Denis Barbier <barbier@linuxfr.org>
- Martin Quinson <martin.quinson@tuxfamily.org>
+ Martin Quinson (mquinson#debian.org)
 
 =head1 COPYRIGHT AND LICENSE
 
  Copyright (c) 1995 by David Megginson <dmeggins@aix1.uottawa.ca>
- Copyright 2002 by SPI, inc.
+ Copyright 2002, 2003, 2004, 2005 by SPI, inc.
 
 This program is free software; you may redistribute it and/or modify it
-under the terms of GPL (see COPYING file).
+under the terms of GPL (see the COPYING file).
