@@ -90,7 +90,11 @@ use Encode::Guess;
 # hash of known commands and environments, with parsing sub.
 # See end of this file
 use vars qw(%commands %environments);
+# hash to describe the number of parameters and which one have to be
+# translated. Used by generic commands
 our %command_parameters = ();
+# hash to describe the separators of environments.
+our %env_separators =();
 
 # The escape character used to introduce commands.
 our $RE_ESCAPE = "\\\\";
@@ -200,6 +204,24 @@ instead of either translated or untranslated.
 
 Indicates that the I<env1> environment should be handled by I<function1>.
 
+=item % po4a: separator <env> "<regex>"
+
+Indicates that an environment should be split according to the given
+regular expression.
+
+The regular expression is delimited by quotes.
+It should not create any backreference.
+You should use (?:) if you need a group.
+It may also need some escapes.  
+
+For example, the LaTeX module uses the "(?:&|\\\\)" regular expression to
+translate separately each cell of a table (lines are separated by '\\' and
+cells by '&').
+
+The notion of environment is expended to the type displayed in the PO file.
+This can be used to split on "\\\\" in the first mandatory argument of the
+title command.  In this case, the environment is title{#1}.
+
 =back
 
 See the B<INTERNAL FUNCTIONS> section for the list of function which could be
@@ -368,7 +390,7 @@ these commands.
 This indicate if a variant is used.  For example, an asterisk (*) can
 be added at the end of sections command to specify that they should
 not be numbered.  In this case, this field will contain "*".  If there
-is not variant, the field is an empty string.
+is no variant, the field is an empty string.
 
 =item An array of optional arguments
 
@@ -420,7 +442,11 @@ sub get_leading_command {
                         $opt .= $2
                     }
                 } else {
-                    die sprintf "un-balanced [";
+                    die wrap_ref_mod($self->{ref},
+                                     "po4a::tex",
+                                     dgettext("po4a", "un-balanced %s in '%s'"),
+                                     "[",
+                                     $buffer);
                 }
             }
             push @opts, $opt;
@@ -446,7 +472,11 @@ sub get_leading_command {
                         $arg .= $2;
                     }
                 } else {
-                    die sprintf "un-balanced {";
+                    die wrap_ref_mod($self->{ref},
+                                     "po4a::tex",
+                                     dgettext("po4a", "un-balanced %s in '%s'"),
+                                     "{",
+                                     $buffer);
                 }
             }
             push @args, $arg;
@@ -496,7 +526,11 @@ sub get_trailing_command {
                      $arg = $2.$arg;
                  }
             } else {
-                die sprintf "un-balanced }";
+                die wrap_ref_mod($self->{ref},
+                                 "po4a::tex",
+                                 dgettext("po4a", "un-balanced %s in '%s'"),
+                                 "}",
+                                 $buffer);
             }
         }
         unshift @args, $arg;
@@ -523,7 +557,11 @@ sub get_trailing_command {
                      $opt = $2.$opt;
                  }
             } else {
-                die sprintf "un-balanced ]";
+                die wrap_ref_mod($self->{ref},
+                                 "po4a::tex",
+                                 dgettext("po4a", "un-balanced %s in '%s'"),
+                                 "]",
+                                 $buffer);
             }
         }
         unshift @opts, $opt;
@@ -631,19 +669,21 @@ sub translate_buffer {
 
     # remove comments from the buffer.
     # Comments are stored in an array and shown as comments in the PO.
-    while ($buffer =~ m/((?<!\\)(?:\\\\)*)%([^\n]*)(\n[ \t]*)(.*)$/s) { # removed |$ at the end
+    while ($buffer =~ m/((?<!\\)(?:\\\\)*)%([^\n]*)(\n[ \t]*)(.*)$/s) {
+        my $comment = $2;
         my $end = "";
         if ($4 =~ m/^\n/s and $buffer !~ m/^%/s) {
             # a line with comments, followed by an empty line.
             # Keep the empty line, but remove the comment.
-            # This is an empiracal euristic, but seems to work;)
+            # This is an empirical heuristic, but seems to work;)
             $end = "\n";
         }
-        if (defined $2 and $2 !~ /^\s*$/) {
-            push @comments, $2;
+        if (defined $comment and $comment !~ /^\s*$/s) {
+            push @comments, $comment;
         }
-        $buffer    =~ s/((?<!\\)(?:\\\\)*)%([^\n]*)(\n[ \t]*)/$1$end/s;
+        $buffer =~ s/((?<!\\)(?:\\\\)*)%([^\n]*)(\n[ \t]*)/$1$end/s;
     }
+    
 
     # translate leading commands.
     do {
@@ -674,7 +714,10 @@ sub translate_buffer {
                 }
                 $translated_buffer .= $spaces;
             } else {
-                die sprintf("unknown command: '%s'", $command)."\n"
+                die wrap_ref_mod($self->{ref},
+                                 "po4a::tex",
+                                 dgettext("po4a", "Unknown command: '%s'"),
+                                 $command);
             }
         } else {
             $buffer = $spaces.$buffer;
@@ -699,6 +742,28 @@ sub translate_buffer {
     } while (length($command));
 
     # Now, $buffer is just a block that can be translated.
+
+    # environment specific treatment
+    if (@env and defined $env_separators{$env[-1]}) {
+        my $re_separator = $env_separators{$env[-1]};
+        my $buf_begin = "";
+        while ($buffer =~ m/^(.*?)(\s*$re_separator\s*)(.*)$/s) {
+            my ($begin, $sep, $end) = ($1, $2, $3);
+            $buf_begin .= $begin;
+            if (is_closed($buf_begin)) {
+                my $t = "";
+                ($t, @env) = translate_buffer($self, $buf_begin, @env);
+                $translated_buffer .= $t.$sep;
+                $buf_begin = "";
+            } else {
+                # the command is in a command argument
+                $buf_begin .= $sep;
+            }
+            $buffer = $end;
+        }
+    }
+
+    # finally, translate
     if (length($buffer)) {
         my $wrap = 1;
         my ($e1, $e2);
@@ -735,7 +800,10 @@ sub translate_buffer {
                                                $opts,$args,\@env);
             $translated_buffer .= $t.$spaces;
         } else {
-            die sprintf("unknown command: '%s'", $command)."\n";
+            die wrap_ref_mod($self->{ref},
+                             "po4a::tex",
+                             dgettext("po4a", "Unknown command: '%s'"),
+                             $command);
         }
     }
 
@@ -799,6 +867,7 @@ sub read_file {
                     $include = 0;
                     $begin .= "\\$tag"."{$newfilename}";
                     $textline = $end;
+                    last;
                 }
             }
             if ($include and ($tag eq "include")) {
@@ -867,7 +936,7 @@ newcommands).
 =cut
 
 sub parse_definition_file {
-    my ($self,$filename)=@_;
+    my ($self,$filename,$only_try)=@_;
 
     foreach (($my_dirname,
               defined($ENV{"TEXINPUTS"})?
@@ -879,9 +948,15 @@ sub parse_definition_file {
         }
     }
 
-    open (IN,"<$filename")
-        || die wrap_mod("po4a::tex",
+    if (! open (IN,"<$filename")) {
+        warn wrap_mod("po4a::tex",
             dgettext("po4a", "Can't open %s: %s"), $filename, $!);
+        if (defined $only_try && $only_try) {
+            return;
+        } else {
+            exit 1;
+        }
+    }
     while (<IN>) {
         if (/^\s*%\s*po4a\s*:/) {
             parse_definition_line($self, $_);
@@ -899,7 +974,7 @@ See the INLINE CUSTOMIZATION section for more details.
 
 sub parse_definition_line {
     my ($self,$line)=@_;
-    $line =~ s/^\s*%\s+po4a\s*:\s*//;
+    $line =~ s/^\s*%\s*po4a\s*:\s*//;
 
     if ($line =~ /^command\s+(\*?)(\w+)\s+(.*)$/) {
         my $command = $2;
@@ -914,7 +989,9 @@ sub parse_definition_line {
                     $command_parameters{$command} = $command_parameters{$1};
                 }
             } else {
-                die "Cannot use an alias to the unknown command $2\n";
+                die wrap_mod("po4a::tex",
+                             dgettext("po4a", "Cannot use an alias to the unknown command '%s'"),
+                             $2);
             }
         } elsif ($line =~ /^(-1|\d+),(-1|\d+),(-1|[ 0-9]*),(-1|[ 0-9]*?)\s*$/) {
             register_generic("$command,$1,$2,$3,$4");
@@ -922,7 +999,9 @@ sub parse_definition_line {
             if (defined &$1) {
                 $commands{$command} = \&$1;
             } else {
-                die "Unknown command ($1) for $command\n";
+                die wrap_mod("po4a::tex",
+                             dgettext("po4a", "Unknown function (%s) for '%s'"),
+                             $1, $command);
             }
         }
     } elsif ($line =~ /^environment\s+(\w+)\s+(.*)$/) {
@@ -932,9 +1011,15 @@ sub parse_definition_line {
             if (defined &$1) {
                 $environments{$env} = \&$1;
             } else {
-                die "Unknown environment ($1) for $env\n";
+                die wrap_mod("po4a::tex",
+                             dgettext("po4a", "Unknown function (%s) for '%s'"),
+                             $1, $env);
             }
         }
+    } elsif ($line =~ /^separator\s+(\w+(?:\[#[0-9]+\]))\s+\"(.*)\"\s*$/) {
+        my $env = $1; # This is not necessarily an environment.
+                      # It can also be smth like 'title[#1]'.
+        $env_separators{$env} = $2
     }
 }
 
@@ -1166,7 +1251,9 @@ $commands{'begin'}= sub {
         ($t, @e) = &{$environments{$envir}}($self,$command,$variant,
                                             $opts,$args,$env);
     } else {
-        die wrap_mod("po4a::tex", "unknown environment: '%s'", $args->[0]);
+        die wrap_mod("po4a::tex",
+                     dgettext("po4a", "unknown environment: '%s'"),
+                     $args->[0]);
     }
 
     print "($t, @e)\n"
@@ -1183,8 +1270,9 @@ $commands{'end'}= sub {
     if (!@$env || @$env[-1] ne $args->[0]) {
         # a begin may have been hidden in the middle of a translated
         # buffer. FIXME: Just warn for now.
-        warn wrap_mod("po4a::tex", "unmatched end of environment '%s'",
-                     $args->[0]);
+        warn wrap_mod("po4a::tex",
+                      dgettext("po4a", "unmatched end of environment '%s'"),
+                      $args->[0]);
     } else {
         pop @$env;
     }
@@ -1204,14 +1292,19 @@ sub generic_command {
     my ($t,@e)=("",());
 
     # check number of arguments
-    die sprintf("wrong number of optional arguments for command $command\n")
+    die wrap_mod("po4a::tex",
+                 dgettext("po4a", "wrong number of optional arguments for command '%s'"),
+                 $command)
         if (    scalar($command_parameters{$command}{'nb_opts'}) lt scalar(@$opts)
             and $command_parameters{$command}{'nb_opts'} ne -1);
     if (    $command_parameters{$command}{'nb_args'} ne scalar(@$args)
         and $command_parameters{$command}{'nb_args'} ne -1) {
         unless (    $command_parameters{$command}{'nb_args'} eq (scalar(@$args) - 1)
                 and !length(@$args[-1])) {
-    die sprintf("wrong number of arguments for command $command\n");
+            die wrap_mod("po4a::tex",
+                         dgettext("po4a",
+                                  "wrong number of arguments for command '%s'"),
+                         $command);
         }
     }
 
@@ -1232,7 +1325,7 @@ sub generic_command {
             }
         }
         if ($have_to_be_translated) {
-            ($t, @e) = translate_buffer($self,$opt,(@$env,$command."[#$arg]"));
+            ($t, @e) = translate_buffer($self,$opt,(@$env,$command."{#$arg}"));
         } else {
             $t = $opt;
         }
@@ -1306,8 +1399,6 @@ sub push_environment {
 $environments{'verbatim'} = \&push_environment;
 $environments{'document'} = \&push_environment;
 
-# TODO: a tabular environment to translate cells separately
-
 ####################################
 ### INITIALIZATION OF THE PARSER ###
 ####################################
@@ -1320,6 +1411,11 @@ sub initialize {
     $self->{options}{'exclude_include'}='';
     $self->{options}{'no_wrap'}='';
     $self->{options}{'debug'}='';
+    $self->{options}{'verbose'}='';
+
+    %debug = ();
+    # FIXME: %commands and %separated should also be restored to their
+    #        default values.
 
     foreach my $opt (keys %options) {
         if ($options{$opt}) {
@@ -1395,9 +1491,10 @@ It was tested on a book and with the Python documentation.
 
 =over 4
 
-=item tabular environment
+=item other categories
 
-A tabular environment would have to translate every cell separately.
+A verbatim category may be needed to indicate that po4a should not attempt
+to rewrap lines, and that percent signs do not introduce any comment.
 
 =item Others
 
